/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


import java.awt.Color;
import java.awt.Graphics2D;
import java.io.IOException;
import types.DimensionFenetre;
import types.NatureObjet;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;


public abstract class CLieuDiscretDeBase extends CElementGraphique {
	CPointAncetre pointATracer;
  public int nombreDePoints;
	CSousListeObjets listeElementsAncetres;
	int[] abscissesCoordonneesPoints;
	int[] ordonneesCoordonneesPoints;
  int indiceDernierPointTrace;
	CPointAncetre point1; // Point servant au trac
	public int motif; // le motif du point servant au trac
  
  /** Creates new CLieuDiscretDeBase */
  public CLieuDiscretDeBase() {
  }
  
  // Constructeur pour clonage
  public CLieuDiscretDeBase(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
        Color couleurInit, boolean masqueInit, CPointAncetre pointATracerInit, int nombreDePointsInit, int motifInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, 0d, 0d, masqueInit);
    motif = motifInit;
    pointATracer = pointATracerInit;
    nombreDePoints = nombreDePointsInit;
    // Il faut crer les tableaux de coordonnes
    abscissesCoordonneesPoints = new int[nombreDePoints];
    ordonneesCoordonneesPoints = new int[nombreDePoints];
    point1 = new CPointBase(liste);
    point1.donneMotif(motif);
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Cette liste sera finie d'tre valide dans setClone()
    listeElementsAncetres = new CSousListeObjets();    
  }

  /**
   * Fonction appele lorsque le nombre de points du lieu a chang
   */
  public void changeCaracteristiques() {
    abscissesCoordonneesPoints = new int[nombreDePoints];
    ordonneesCoordonneesPoints = new int[nombreDePoints];  
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    motif = ((CLieuDiscretDeBase) ptel).motif; 
    indiceDernierPointTrace = ((CLieuDiscretDeBase)ptel).indiceDernierPointTrace;
    // Il faut copier toutes les coordonnes
    for (int i = 0; i < indiceDernierPointTrace; i ++)
    {
      abscissesCoordonneesPoints[i] = ((CLieuDiscretDeBase) ptel).abscissesCoordonneesPoints[i];
      ordonneesCoordonneesPoints[i] = ((CLieuDiscretDeBase) ptel).ordonneesCoordonneesPoints[i];
    }
  }

  @Override
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || pointATracer.dependDe(p);
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    
    pDC.setColor(couleur);
    point1.donneMotif(motif);
    point1.donneCouleur(couleur);
    for (int i = 0; i <= indiceDernierPointTrace; i ++)
    {
      point1.placeEn(abscissesCoordonneesPoints[i],ordonneesCoordonneesPoints[i]);
      point1.trace(pDC, couleurFond, transparence);
    }
  }  
 
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
    if (!existe || masque) return -1;
    else {
    	for (int i = 0; i <= indiceDernierPointTrace; i ++)
    	{
    		double d = Kernel.distancePointPoint(xp,yp,abscissesCoordonneesPoints[i],ordonneesCoordonneesPoints[i]);
    		if (d < CListeObjets.distanceMiniPourEtreProche) return d;
    	}
    	return -1;
    }
  }
  
  /**
   * Fonction renvoyant true si le lieu existe et au moins un des
   * points de ce lieu est visible. Utilis pour le trac automatique de courbes
   */
  public boolean dansFenetre(DimensionFenetre dimfen) {
  	if (!existe) return false;
  	boolean resultat = false;
  	for (int i = 0;i <= indiceDernierPointTrace;i ++)
  	{
  		resultat = resultat || dimfen.dansFenetre(abscissesCoordonneesPoints[i], ordonneesCoordonneesPoints[i]);
  	}
  	return resultat;
	}
  
  
  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (pointATracer== ancienPoint) pointATracer = nouveauPoint;
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NLieuDiscret;
  }  
  
  @Override
  public boolean modifiableParMenu() {
  	return true;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    point1 = new CPointBase(list);
    int ind1 = inps.readCInt();
    pointATracer = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    nombreDePoints = inps.readCInt();
    motif = inps.readByte();
    // Il faut crer les tableaux de coordonnes
    // On rserve une coordonne supplmentaire pour le cas d'un lieu ferm
    abscissesCoordonneesPoints = new int[nombreDePoints];
    ordonneesCoordonneesPoints = new int[nombreDePoints];
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la lsite mre
    // Ligne suivante  revoir version 3.4.2
    // listeElementsAncetres = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
    listeElementsAncetres = new CSousListeObjets();
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    point1 = new CPointBase(list);
    int ind1 = inps.readInt();
    pointATracer = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    nombreDePoints = inps.readInt();
    motif = inps.readByte();
    // Il faut crer les tableaux de coordonnes
    // On rserve une coordonne supplmentaire pour le cas d'un lieu ferm
    abscissesCoordonneesPoints = new int[nombreDePoints];
    ordonneesCoordonneesPoints = new int[nombreDePoints];
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la lsite mre
    // Ligne suivante  revoir version 3.4.2
    // listeElementsAncetres = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
    listeElementsAncetres = new CSousListeObjets();
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(pointATracer);
    oups.writeInt(ind1);
    oups.writeInt(nombreDePoints);
    oups.writeByte(motif);
  }
}
